#!/usr/bin/perl

#┌─────────────────────────────────
#│ ASKA BBS : aska.cgi - 2022/03/10
#│ copyright (c) kentweb, 1997-2021
#│ http://www.kent-web.com/
#└─────────────────────────────────
# REBUILD(改造)
#┌─────────────────────────────────
#│  ASKA BBS v8.23 2022/03/10
#│  Edited by Hamel(TRANSFORM) Since 2012
#│  https://www.transformsite.com/
#└─────────────────────────────────
# モジュール宣言
use strict;
use CGI::Carp qw(fatalsToBrowser);
use lib "./lib";
use CGI::Minimal;

# 設定ファイル認識
require "./init.cgi";
my %cf = set_init();

# データ受理
CGI::Minimal::max_read_size($cf{maxdata});
my $cgi = CGI::Minimal->new;
error('容量オーバー') if ($cgi->truncated);
my %in = parse_form($cgi);

# アクセス制限
passwd(%in) if ($cf{enter_pwd} ne '');

# 処理分岐
if ($in{mode} eq 'recv') { recv_data(); }
if ($in{mode} eq 'recv_read') { recv_read_data(); }
if ($in{mode} eq 'dele') { dele_data(); }
if ($in{mode} eq 'edit') { edit_data(); }
if ($in{mode} eq 'gate_form') { gate_form(); }
if ($in{mode} eq 'find') { find_data(); }
if ($in{mode} eq 'note') { note_page(); }
if ($in{mode} eq 'past') { past_data(); }
if ($in{mode} eq 'form') { form_page(); }
if ($in{mode} eq 'album') { album(); }
if ($in{mode} eq 'album_past') { album_past(); }
if ($in{res}) { read_log($in{res}); }
bbs_list();

#-----------------------------------------------------------
#  記事表示
#-----------------------------------------------------------
sub bbs_list {
	my %err = @_;
	
	# 削除フォーム
	if ($in{del}) { del_form(); }
	if ($in{edit}) { edit_form(); }
	
	# レス処理
	$in{res} =~ s/\D//g;
	if ($in{res} > 0) {
		my ($flg,$rsub,$rcom);
		open(IN,"$cf{logfile}") or error("open err: $cf{logfile}");
		while (<IN>) {
			my ($no,$sub,$com) = (split(/<>/))[0,4,5];
			if ($in{res} == $no) {
				$flg++;
				$rsub = $sub;
				$rcom = $com;
				last;
			}
		}
		close(IN);
		
		if (!$flg) { error("該当記事が見つかりません"); }
		
		$rsub =~ s/^Re://g;
		$rsub =~ s/\[\d+\]\s?//g;
		$rsub = "Re:[$in{res}] $rsub";
		$rcom = "&gt; $rcom";
		$rcom =~ s|<br( /)?>|\n&gt; |ig;
		
		$in{sub} = $rsub;
		$in{comment} = $rcom;
	}
	
	# ページ数定義
	my $pg = $in{pg} || 0;
	
	# データオープン
	my ($i,@log);
	open(IN,"$cf{logfile}") or error("open err: $cf{logfile}");
	while (<IN>) {
		$i++;
		next if ($i < $pg + 1);
		next if ($i > $pg + $cf{pg_max});
		
		push(@log,$_);
	}
	close(IN);
	
	# 繰越ボタン作成
	my $page_btn = make_pager($i,$pg);
	
	# クッキー取得
	my @cook = get_cookie();
	if ($cook[2] eq '') { $cook[2] = 'http://'; }

	# home or logoff
	my $home = $cf{enter_pwd} eq '' ? $cf{homepage} : "$cf{bbs_cgi}?mode=logoff";

	# カウンタ
	my $counter = bbs_count() if ($cf{counter});
	
	# テンプレート読込
	open(IN,"$cf{tmpldir}/bbs.html") or error("open err: bbs.html");
	my $tmpl = join('',<IN>);
	close(IN);
	
	# 文字置換
	$tmpl =~ s/<!-- past_begin -->.+<!-- past_end -->//s if ($cf{pastkey} == 0);
	$tmpl =~ s/!([a-z]+_cgi)!/$cf{$1}/g;
	$tmpl =~ s/!homepage!/$home/g;
	$tmpl =~ s/<!-- page_btn -->/$page_btn/g;
	$tmpl =~ s/!new_mark!/$cf{new_mark}/g;
	$tmpl =~ s/!new_time!/$cf{new_time}/g;
	$tmpl =~ s/!bbs_title!/掲示板 - $cf{bbs_title} PLUS/g;
	$tmpl =~ s/!counter!/$counter/g;
	$tmpl =~ s/!pg!/$pg/g;
	$tmpl =~ s|!icon:(\w+\.\w+)!|<img src="$cf{cmnurl}/$1" class="icon" alt="$1">|g;
	$tmpl =~ s/!cmnurl!/$cf{cmnurl}/g;
	
	#if ($in{job_bak}) {
	#	$in{comment} =~ s/\t/\n/g;
	#	$tmpl =~ s/!(name|email|url|sub|comment|pwd)!/$in{$1}/g;
	#} else {
	#	$tmpl =~ s/!name!/$err{name} ne '' ? $in{name} : $cook[0]/e;
	#	$tmpl =~ s/!email!/$err{eml} ne '' ? $in{email} : $cook[1]/e;
	#	$tmpl =~ s/!url!/$err{url} ne '' ? $in{url} : $cook[2]/e;
	#	$tmpl =~ s/!sub!/$in{sub}/;
	#	$tmpl =~ s/!comment!/$in{comment}/;
	#	$tmpl =~ s/!pwd!//;
	#}
	
	# 入力エラー時
	#if (%err > 0) {
	#	for (keys %err) { $tmpl =~ s|<!-- err:$_ -->|<div class="err-col">$err{$_}</div>|; }
	#}
	
	# テンプレート分割
	my ($head,$loop,$foot) = $tmpl =~ m|(.+)<!-- loop -->(.+?)<!-- /loop -->(.+)|s
			? ($1,$2,$3) : error("テンプレート不正");
	
	# ヘッダ表示
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;
	
	# ループ部
	for (@log) {
		my ($no,$date,$name,$eml,$sub,$com,$url,$host,$pw,$tim,$ex,$w,$h,$chk) = split(/<>/);
		$name = qq|<a href="mailto:$eml">$name</a>| if ($eml);
		$com = autolink($com) if ($cf{autolink});
		$com =~ s/([>]|^)(&gt;[^<]*)/$1<span style="color:$cf{ref_col}">$2<\/span>/g if ($cf{ref_col});
		
		# 所定時間以内の投稿は[NEWマーク]表示
		my $time = time;
		my $newsign;
		if ($time - $tim > $cf{new_time} * 3600) { $newsign = ""; }
		else { $newsign = $cf{new_mark}; }

		# 添付あり
		if ($ex) {
			# 確認待ち
			if ($cf{imgcheck} && $chk != 1) {
				my $size = get_size("$cf{imgdir}/$no$ex");
				$com .= qq|<p>[添付]: 認証待ち (<b style="color:#FF9966">$size</b>)</p>|;

			# 添付画像
			} elsif ($ex =~ /jpg$|gif$|png$/) {
				my $op;
				if (-f "$cf{imgdir}/$no-s$ex") {
					$op = qq|src="$cf{imgurl}/$no-s$ex"|;
				} else {
					($w,$h) = resize($w,$h);
					$op = qq|src="$cf{imgurl}/$no$ex" width="$w" height="$h"|;
				}

			# 画像はコメントの下
			if ($cf{imgpoint} == 1) {
				$com .= qq|<div class="ext"><a href="$cf{imgurl}/$no$ex" data-lightbox="img$no" data-title="$sub"><img $op border="0" class="snapshot"></a></div>|;
			# 画像はコメントの左（廻り込み）
			} else {
				$com = qq|<a href="$cf{imgurl}/$no$ex" data-lightbox="img$no" data-title="$sub"><img $op border="0" align="left" hspace="18" class="snapshot"></a>$com<br clear="all">|;
			}

			# 画像以外
			} else {
				my $size = get_size("$cf{imgdir}/$no$ex");
				$com .= qq|<p>添付: <a href="$cf{imgurl}/$no$ex" target="_blank"><img src="$cf{ico_clip}" class="clip"></a> (<b style="color:#FF9966">$size</b>)</p>|;
			}
		}
		
		my $tmp = $loop;
		$tmp =~ s/!num!/$no/g;
		$tmp =~ s/!sub!/$sub $newsign/g;
		$tmp =~ s/!name!/$name/g;
		$tmp =~ s/!date!/$date/g;
		$tmp =~ s/!comment!/$com/g;
		$tmp =~ s/!bbs_cgi!/$cf{bbs_cgi}/g;
		$tmp =~ s/!home!/home_link($url)/e;
		print $tmp;
	}
	
	# フッタ
	footer($foot);
}

#-----------------------------------------------------------
#  記事閲覧（返信フォーム）
#-----------------------------------------------------------
sub read_log {
	my %err = @_;

	# レス処理
	$in{res} =~ s/\D//g;

	my %res;
	if ($in{res}) {
		my $flg;
		open(IN,"$cf{logfile}") or &error("open err: $cf{logfile}");
		while (<IN>) {
			my ($no,$sub,$com) = (split(/<>/))[0,4,5];
			if ($in{res} == $no) {
				$flg++;
				$res{sub} = $sub;
				$res{com} = $com;
				last;
			}
		}
		close(IN);

		if (!$flg) { &error("該当記事が見つかりません"); }

		$res{sub} =~ s/^Re://g;
		$res{sub} =~ s/\[\d+\]\s?//g;
		$res{sub} = "Re:[$in{res}] $res{sub}";
		$res{com} = "&gt; $res{com}";
		$res{com} =~ s/<br>/\n&gt; /ig;
	}

	# ページ数定義
	my $pg = $in{pg} || 0;

	# データオープン
	my ($i,@log);
	open(IN,"$cf{logfile}") or &error("open err: $cf{logfile}");
	while (<IN>) {
		$i++;
		next if ($i < $pg + 1);
		next if ($i > $pg + $cf{pg_max});

		push(@log,$_);
	}
	close(IN);

	# 繰越ボタン作成
	my $page_btn = make_pager($i,$pg);

	# クッキー取得
	my @cook = get_cookie();
	if ($cook[2] eq '') { $cook[2] = 'http://'; }

	# テンプレート読込
	open(IN,"$cf{tmpldir}/read.html") or &error("open err: read.html");
	my $tmpl = join('', <IN>);
	close(IN);

	$tmpl =~ s/!cmnurl!/$cf{cmnurl}/g;
	$tmpl =~ s/!pg!/$pg/g;
	$tmpl =~ s|!icon:(\w+\.\w+)!|<img src="$cf{cmnurl}/$1" alt="" class="icon" />|g;

	if ($in{job_bak}) {
		$in{comment} =~ s/\t/\n/g;
		$tmpl =~ s/!(name|email|url|sub|comment|pwd)!/$in{$1}/g;
	} else {
		$tmpl =~ s/!name!/$err{name} ne '' ? $in{name} : $cook[0]/e;
		$tmpl =~ s/!email!/$err{eml} ne '' ? $in{email} : $cook[1]/e;
		$tmpl =~ s/!url!/$err{url} ne '' ? $in{url} : $cook[2]/e;
		#$tmpl =~ s/!sub!/$in{sub}/;
		#$tmpl =~ s/!comment!/$in{comment}/;
		$tmpl =~ s/!pwd!//;
	}
	
	# 入力エラー時
	if (%err > 0) {
		for (keys %err) { $tmpl =~ s|<!-- err:$_ -->|<div class="err-col">$err{$_}</div>|; }
	}

	# 改行復元
	my ($no,$date,$name,$eml,$sub,$com,$url,$host,$pw,$tim,$ex,$w,$h,$chk) = split(/<>/);
	$com =~ s/<br>/\n/g;

	# テンプレート分割
	my ($head,$loop,$foot) = $tmpl =~ m|(.+)<!-- loop -->(.+?)<!-- /loop -->(.+)|s
			? ($1,$2,$3) : error("テンプレート不正");

	# 文字置換
	foreach ($head,$foot) {
		s/!([a-z]+_cgi)!/$cf{$1}/g;
		s/!bbs_title!/$cf{bbs_title}/g;
		s/!sub!/$res{sub}/;
		s/!resno!/$in{res}/;
		s/!comment!/$res{com}/;
		s/<!-- page_btn -->/$page_btn/g;
	}

	# ヘッダ部
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;

	# ループ部
	foreach (@log) {
		my ($no,$date,$name,$eml,$sub,$com,$url,$host,$pw,$tim,$ex,$w,$h,$chk) = split(/<>/);
		$name = qq|<a href="mailto:$eml">$name</a>| if ($eml);
		$com = &autolink($com) if ($cf{autolink});
		$com =~ s/([>]|^)(&gt;[^<]*)/$1<span style="color:$cf{ref_col}">$2<\/span>/g if ($cf{ref_col});

		# 所定時間以内の投稿は[NEWマーク]表示
		my $time = time;
		my $newsign;
		if ($time - $tim > $cf{new_time} * 3600) { $newsign = ""; }
		else { $newsign = $cf{new_mark}; }

		# 添付あり
		if ($ex) {
			# 確認待ち
			if ($cf{imgcheck} && $chk != 1) {
				my $size = get_size("$cf{imgdir}/$no$ex");
				$com .= qq|<p>[添付]: 認証待ち (<b style="color:#FF9966">$size</b>)</p>|;

			# 添付画像
			} elsif ($ex =~ /jpg$|gif$|png$/) {
				my $op;
				if (-f "$cf{imgdir}/$no-s$ex") {
					$op = qq|src="$cf{imgurl}/$no-s$ex"|;
				} else {
					($w,$h) = &resize($w,$h);
					$op = qq|src="$cf{imgurl}/$no$ex" width="$w" height="$h"|;
				}
				#$com .= qq|<div class="ext"><a href="$cf{imgurl}/$no$ex" target="_blank"><img $op border="0" class="snapshot"></a></div>|;

			# 画像はコメントの下
			if ($cf{imgpoint} == 1) {
				$com .= qq|<div class="ext"><a href="$cf{imgurl}/$no$ex"  data-lightbox="img$no" data-title="$sub"><img $op border="0" class="snapshot"></a></div>|;
			# 画像はコメントの左（廻り込み）
			} else {
				$com = qq|<a href="$cf{imgurl}/$no$ex" data-lightbox="img$no" data-title="$sub"><img $op border="0" align="left" hspace="18" class="snapshot"></a>$com<br clear="all">|;
			}

			# 画像以外
			} else {
				my $size = get_size("$cf{imgdir}/$no$ex");
				$com .= qq|<p>添付: <a href="$cf{imgurl}/$no$ex" target="_blank"><img src="$cf{ico_clip}" class="clip"></a> (<b style="color:#FF9966">$size</b>)</p>|;
			}
		}

		my $tmp = $loop;
		$tmp =~ s/!num!/$no/g;
		$tmp =~ s/!sub!/$sub $newsign/g;
		$tmp =~ s/!art-name!/$name/g;
		$tmp =~ s/!date!/$date/g;
		$tmp =~ s/!comment!/$com/g;
		$tmp =~ s/!home!/home_link($url)/e;
		$tmp =~ s/!bbs_cgi!/$cf{bbs_cgi}/g;
		print $tmp;
	}

	# フッタ
	footer($foot);
}

#-----------------------------------------------------------
#  投稿フォーム
#-----------------------------------------------------------
sub form_page {
	my %err = @_;

	# クッキー取得
	my @cook = get_cookie();
	if ($cook[2] eq '') { $cook[2] = 'http://'; }

	# 改行復元
	my ($no,$date,$name,$eml,$sub,$com,$url,$host,$pw,$tim,$ex,$w,$h,$chk) = split(/<>/);
	$com =~ s/<br>/\n/g;

	open(IN,"$cf{tmpldir}/form.html") or &error("open err: form.html");
	my $tmpl = join('', <IN>);
	close(IN);

	if ($in{job_bak}) {
		$in{comment} =~ s/\t/\n/g;
		$tmpl =~ s/!(name|email|url|sub|comment|pwd)!/$in{$1}/g;
	} else {
		$tmpl =~ s/!name!/$err{name} ne '' ? $in{name} : $cook[0]/e;
		$tmpl =~ s/!email!/$err{eml} ne '' ? $in{email} : $cook[1]/e;
		$tmpl =~ s/!url!/$err{url} ne '' ? $in{url} : $cook[2]/e;
		$tmpl =~ s/!sub!/$in{sub}/;
		$tmpl =~ s/!comment!/$in{comment}/;
		$tmpl =~ s/!pwd!//;
	}
	
	# 入力エラー時
	if (%err > 0) {
		for (keys %err) { $tmpl =~ s|<!-- err:$_ -->|<div class="err-col">$err{$_}</div>|; }
	}

	$tmpl =~ s/!([a-z]+_cgi)!/$cf{$1}/g;
	$tmpl =~ s/!bbs_title!/$cf{bbs_title}/g;
	$tmpl =~ s/!name!/$cook[0]/;
	$tmpl =~ s/!email!/$cook[1]/;
	$tmpl =~ s/!url!/$cook[2]/;
	$tmpl =~ s/!sub!/$sub/;
	$tmpl =~ s/!comment!/$com/;
	$tmpl =~ s/!cmnurl!/$cf{cmnurl}/g;

	print "Content-type: text/html; charset=utf-8\n\n";
	print $tmpl;

	# フッタ
	footer();

	exit;
}

#-----------------------------------------------------------
#  記事受付
#-----------------------------------------------------------
sub recv_data {
	# 戻り
	if ($in{job_bak}) { form_page(); }
	
	# 投稿チェック
	check_post();
	
	# 確認画面
	if ($in{job_reg} eq '') {
		conf_form();
	
	# セッション確認
	} else {
		check_ses();
	}
	
	$in{comment} =~ s/\t/<br>/g;
	
	# ホスト取得
	my ($host,$addr) = get_host();
	
	# 削除キー暗号化
	my $pwd = encrypt($in{pwd}) if ($in{pwd} ne "");
	
	# 時間取得
	my $time = time;
	my ($min,$hour,$mday,$mon,$year,$wday) = (localtime($time))[1..6];
	my @wk = ('Sun','Mon','Tue','Wed','Thu','Fri','Sat');
	my $date = sprintf("%04d/%02d/%02d(%s) %02d:%02d",
				$year+1900,$mon+1,$mday,$wk[$wday],$hour,$min);
	
	# 先頭記事読み取り
	open(DAT,"+< $cf{logfile}") or error("open err: $cf{logfile}");
	eval "flock(DAT,2);";
	my $top = <DAT>;
	
	# 重複投稿チェック
	my ($no,$dat,$nam,$eml,$sub,$com,$url,$hos,$pw,$tim) = split(/<>/,$top);
	if ($in{name} eq $nam && $in{comment} eq $com) {
		close(DAT);
		error("二重投稿は禁止です");
	}
	
	# 記事No採番
	$no++;

	# ファイル添付処理
	my ($ext,$w,$h) = upload($no) if ($in{upfile});
	
	# 最大記事数処理
	my $i = 0;
	my (@data,@old);
	seek(DAT, 0, 0);
	while (<DAT>) {
		$i++;
		my ($no,$date,$name,$eml,$sub,$com,$url,$host,$pw,$tim,$ex,$w,$h,$chk) = split(/<>/);

		if ($i <= $cf{maxlog} - 1) {
			push(@data,$_);
		} else {
			push(@old,$_);
		}
	}

	# 更新
	seek(DAT, 0, 0);
	print DAT "$no<>$date<>$in{name}<>$in{email}<>$in{sub}<>$in{comment}<>$in{url}<>$host<>$pwd<>$time<>$ext<>$w<>$h<>0<>\n";
	print DAT @data;
	truncate(DAT, tell(DAT));
	close(DAT);

	# クッキー格納
	set_cookie($in{name},$in{email},$in{url}) if ($in{cookie} == 1) ;

	# 過去ログ更新
	if ($cf{pastkey} && @old > 0) {
		make_pastlog(@old)
	}

	# メール通知
	my $clip = "$no$ext" if ($ext);
	mail_to($date,$host,$clip) if ($cf{mailing} == 1);
	
	# 完了画面
	message("ありがとうございます。記事を受理しました。");
}

#-----------------------------------------------------------
#  返信記事受付
#-----------------------------------------------------------
sub recv_read_data {
	# 戻り
	if ($in{read_bak}) { read_log(); }
	
	# 投稿チェック
	check_post();
	
	# 確認画面
	if ($in{job_reg} eq '') {
		read_conf_form();
	
	# セッション確認
	} else {
		check_ses();
	}
	
	$in{comment} =~ s/\t/<br>/g;
	
	# ホスト取得
	my ($host,$addr) = get_host();
	
	# 削除キー暗号化
	my $pwd = encrypt($in{pwd}) if ($in{pwd} ne "");
	
	# 時間取得
	my $time = time;
	my ($min,$hour,$mday,$mon,$year,$wday) = (localtime($time))[1..6];
	my @wk = ('Sun','Mon','Tue','Wed','Thu','Fri','Sat');
	my $date = sprintf("%04d/%02d/%02d(%s) %02d:%02d",
				$year+1900,$mon+1,$mday,$wk[$wday],$hour,$min);
	
	# 先頭記事読み取り
	open(DAT,"+< $cf{logfile}") or error("open err: $cf{logfile}");
	eval "flock(DAT,2);";
	my $top = <DAT>;
	
	# 重複投稿チェック
	my ($no,$dat,$nam,$eml,$sub,$com,$url,$hos,$pw,$tim) = split(/<>/,$top);
	if ($in{name} eq $nam && $in{comment} eq $com) {
		close(DAT);
		error("二重投稿は禁止です");
	}
	
	# 記事No採番
	$no++;

	# ファイル添付処理
	my ($ext,$w,$h) = upload($no) if ($in{upfile});
	
	# 最大記事数処理
	my $i = 0;
	my (@data,@old);
	seek(DAT, 0, 0);
	while (<DAT>) {
		$i++;
		my ($no,$date,$name,$eml,$sub,$com,$url,$host,$pw,$tim,$ex,$w,$h,$chk) = split(/<>/);

		if ($i <= $cf{maxlog} - 1) {
			push(@data,$_);
		} else {
			push(@old,$_);
		}
	}

	# 更新
	seek(DAT, 0, 0);
	print DAT "$no<>$date<>$in{name}<>$in{email}<>$in{sub}<>$in{comment}<>$in{url}<>$host<>$pwd<>$time<>$ext<>$w<>$h<>0<>\n";
	print DAT @data;
	truncate(DAT, tell(DAT));
	close(DAT);

	# クッキー格納
	set_cookie($in{name},$in{email},$in{url}) if ($in{cookie} == 1) ;

	# 過去ログ更新
	if ($cf{pastkey} && @old > 0) {
		make_pastlog(@old)
	}

	# メール通知
	my $clip = "$no$ext" if ($ext);
	mail_to($date,$host,$clip) if ($cf{mailing} == 1);
	
	# 完了画面
	message("ありがとうございます。記事を受理しました。");
}

#-----------------------------------------------------------
#  確認画面
#-----------------------------------------------------------
sub conf_form {
	# ホスト取得
	my ($host,$addr) = get_host();
	
	open(IN,"$cf{logfile}") or error("open err: $cf{logfile}");
	my $top = <IN>;
	close(IN);
	
	chomp $top;
	my ($no,$dat,$nam,$eml,$sub,$com,$url,$hos,$pw,$tim,$ex,$w,$h,$chk) = split(/<>/,$top);
	if ($in{name} eq $nam && $in{comment} eq $com) {
		error("二重投稿は禁止です");
	}
	if ($host eq $hos && time - $tim < $cf{wait_time}) {
		error("連続投稿は$cf{wait_time}秒以上空けてください");
	}
	
	# 画像認証用
	my @dig = (0 .. 9);
	my $dig;
	for (1 .. $cf{cap_len}) { $dig .= $dig[int(rand(@dig))]; }
	
	# セッション文字
	my @wd = (0 .. 9, 'a' .. 'z', 'A' .. 'Z', '_');
	my $ses;
	for (1 .. 25) { $ses .= $wd[int(rand(@wd))]; }
	
	# セッションファイル記録
	my $now = time;
	my @log;
	open(DAT,"+< $cf{datadir}/conf.cgi") or error("open err: $cf{datadir}/conf.cgi");
	eval "flock(DAT,2);";
	while(<DAT>) {
		my ($time,$rand,$fig) = split(/\t/);
		next if ($now - $time > $cf{cap_time}*60);
		
		push(@log,$_);
	}
	unshift(@log,"$now\t$ses\t$dig\t$addr\n");
	seek(DAT,0,0);
	print DAT @log;
	truncate(DAT,tell(DAT));
	close(DAT);
	
	# 引数
	my $hid;
	for (qw(name email sub url pwd cookie)) {
		$hid .= qq|<input type="hidden" name="$_" value="$in{$_}">\n|;
	}
	my $com = $in{comment};
	$com =~ s/<br>/\t/g;
	$hid .= qq|<input type="hidden" name="comment" value="$com">\n|;
	$hid .= qq|<input type="hidden" name="ses" value="$ses">\n|;
	
	my $pwd = $in{pwd};
	$pwd =~ s/./*/g;
	
	# テンプレート読み込み
	open(IN,"$cf{tmpldir}/conf.html") or die;
	my $tmpl = join('',<IN>);
	close(IN);
	
	if ($cf{use_captcha} == 0) {
		$tmpl =~ s|<!-- captcha -->.+?<!-- /captcha -->||s;
	}

	$tmpl =~ s/!(bbs_cgi|cmnurl|bbs_title)!/$cf{$1}/g;
	$tmpl =~ s/!(name|email|comment|url|sub)!/$in{$1}/g;
	$tmpl =~ s/!pwd!/$pwd/g;
	$tmpl =~ s/!cookie!/$in{cookie} == 1 ? 'する' : 'しない'/e;
	$tmpl =~ s/<!-- hidden -->/$hid/;
	$tmpl =~ s|!icon:(\w+\.\w+)!|<img src="$cf{cmnurl}/$1" class="icon" alt="$1">|g;
	$tmpl =~ s|!captcha!|<img src="$cf{captcha_cgi}?$ses" class="icon" alt="投稿キー">|;

	# 画面表記
	print "Content-type: text/html; charset=utf-8\n\n";
	print $tmpl;
	exit;
}

#-----------------------------------------------------------
#  返信確認画面
#-----------------------------------------------------------
sub read_conf_form {
	# ホスト取得
	my ($host,$addr) = get_host();
	
	open(IN,"$cf{logfile}") or error("open err: $cf{logfile}");
	my $top = <IN>;
	close(IN);
	
	chomp $top;
	my ($no,$dat,$nam,$eml,$sub,$com,$url,$hos,$pw,$tim) = split(/<>/,$top);
	if ($in{name} eq $nam && $in{comment} eq $com) {
		error("二重投稿は禁止です");
	}
	if ($host eq $hos && time - $tim < $cf{wait_time}) {
		error("連続投稿は$cf{wait_time}秒以上空けてください");
	}
	
	# 画像認証用
	my @dig = (0 .. 9);
	my $dig;
	for (1 .. $cf{cap_len}) { $dig .= $dig[int(rand(@dig))]; }
	
	# セッション文字
	my @wd = (0 .. 9, 'a' .. 'z', 'A' .. 'Z', '_');
	my $ses;
	for (1 .. 25) { $ses .= $wd[int(rand(@wd))]; }
	
	# セッションファイル記録
	my $now = time;
	my @log;
	open(DAT,"+< $cf{datadir}/conf.cgi") or error("open err: $cf{datadir}/conf.cgi");
	eval "flock(DAT,2);";
	while(<DAT>) {
		my ($time,$rand,$fig) = split(/\t/);
		next if ($now - $time > $cf{cap_time}*60);
		
		push(@log,$_);
	}
	unshift(@log,"$now\t$ses\t$dig\t$addr\n");
	seek(DAT,0,0);
	print DAT @log;
	truncate(DAT,tell(DAT));
	close(DAT);
	
	# 引数
	my $hid;
	for (qw(name email sub url pwd cookie)) {
		$hid .= qq|<input type="hidden" name="$_" value="$in{$_}">\n|;
	}
	my $com = $in{comment};
	$com =~ s/<br>/\t/g;
	$hid .= qq|<input type="hidden" name="comment" value="$com">\n|;
	$hid .= qq|<input type="hidden" name="ses" value="$ses">\n|;
	
	my $pwd = $in{pwd};
	$pwd =~ s/./*/g;
	
	# テンプレート読み込み
	open(IN,"$cf{tmpldir}/read-conf.html") or die;
	my $tmpl = join('',<IN>);
	close(IN);

	if ($cf{use_captcha} == 0) {
		$tmpl =~ s|<!-- captcha -->.+?<!-- /captcha -->||s;
	}
	
	$tmpl =~ s/!(bbs_cgi|cmnurl|bbs_title)!/$cf{$1}/g;
	$tmpl =~ s/!(name|email|comment|url|sub)!/$in{$1}/g;
	$tmpl =~ s/!pwd!/$pwd/g;
	$tmpl =~ s/!cookie!/$in{cookie} == 1 ? 'する' : 'しない'/e;
	$tmpl =~ s/<!-- hidden -->/$hid/;
	$tmpl =~ s|!icon:(\w+\.\w+)!|<img src="$cf{cmnurl}/$1" class="icon" alt="$1">|g;
	$tmpl =~ s|!captcha!|<img src="$cf{captcha_cgi}?$ses" class="icon" alt="投稿キー">|;

	# 画面表記
	print "Content-type: text/html; charset=utf-8\n\n";
	print $tmpl;
	exit;
}

#-----------------------------------------------------------
#  ワード検索
#-----------------------------------------------------------
sub find_data {
	# 条件
	$in{cond} =~ s/\D//g;
	$in{word} =~ s|<br>||g;
	
	# 検索実行
	my @log = search($in{word},$in{cond}) if ($in{word} ne '');
	my $hits = @log;
	
	# テンプレート読み込み
	open(IN,"$cf{tmpldir}/find.html") or error("open err: find.html");
	my $tmpl = join('',<IN>);
	close(IN);
	
	# 文字置換
	$tmpl =~ s/!(bbs_cgi|cmnurl|bbs_title)!/$cf{$1}/g;
	$tmpl =~ s/<!-- op_cond -->/op_cond()/e;
	$tmpl =~ s/!word!/$in{word}/;
	$tmpl =~ s|!icon:(\w+\.\w+)!|<img src="$cf{cmnurl}/$1" class="icon" alt="$1">|g;
	if ($in{word} eq '') {
		$tmpl =~ s|<!-- hits_msg -->.+<!-- /hits_msg -->||s;
	} else {
		$tmpl =~ s/!hits!/$hits/;
	}
	
	# テンプレート分割
	my ($head,$loop,$foot) = $tmpl =~ m|(.+)<!-- loop -->(.+)<!-- /loop -->(.+)|s
			? ($1,$2,$3) : error("テンプレート不正");
	
	# ヘッダ部
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;
	
	# ループ部
	for (@log) {
		my ($no,$date,$name,$eml,$sub,$com,$url,$host,$pw,$tim,$ex,$w,$h,$chk) = split(/<>/);
		$name = qq|<a href="mailto:$eml">$name</a>| if ($eml);
		$com  = autolink($com) if ($cf{autolink});
		$com  =~ s/([>]|^)(&gt;[^<]*)/$1<span style="color:$cf{ref_col}">$2<\/span>/g if ($cf{ref_col});

		# 添付あり
		if ($ex) {
			# 確認待ち
			if ($cf{imgcheck} && $chk != 1) {
				my $size = get_size("$cf{imgdir}/$no$ex");
				$com .= qq|<p>[添付]: 認証待ち (<b style="color:#FF9966">$size</b>)</p>|;

			# 添付画像
			} elsif ($ex =~ /jpg$|gif$|png$/) {
				my $op;
				if (-f "$cf{imgdir}/$no-s$ex") {
					$op = qq|src="$cf{imgurl}/$no-s$ex"|;
				} else {
					($w,$h) = resize($w,$h);
					$op = qq|src="$cf{imgurl}/$no$ex" width="$w" height="$h"|;
				}

			# 画像はコメントの下
			if ($cf{imgpoint} == 1) {
				$com .= qq|<div class="ext"><a href="$cf{imgurl}/$no$ex" data-lightbox="img$no" data-title="$sub"><img $op border="0" class="snapshot"></a></div>|;
			# 画像はコメントの左（廻り込み）
			} else {
				$com = qq|<a href="$cf{imgurl}/$no$ex" data-lightbox="img$no" data-title="$sub"><img $op border="0" align="left" hspace="18" class="snapshot"></a>$com<br clear="all">|;
			}

			# 画像以外
			} else {
				my $size = get_size("$cf{imgdir}/$no$ex");
				$com .= qq|<p>添付: <a href="$cf{imgurl}/$no$ex" target="_blank"><img src="$cf{ico_clip}" class="clip"></a> (<b style="color:#FF9966">$size</b>)</p>|;
			}
		}

		my $tmp = $loop;
		$tmp =~ s/!num!/$no/g;
		$tmp =~ s/!sub!/$sub/g;
		$tmp =~ s/!date!/$date/g;
		$tmp =~ s/!name!/$name/g;
		$tmp =~ s/!home!/home_link($url)/e;
		$tmp =~ s/!comment!/$com/g;
		print $tmp;
	}
	
	# フッタ
	footer($foot);
}

#-----------------------------------------------------------
#  検索実行
#-----------------------------------------------------------
sub search {
	my ($word,$cond) = @_;
	
	# キーワードを配列化
	$word =~ s/　/ /g;
	my @wd = split(/\s+/,$word);
	
	# UTF-8定義
	my $byte1 = '[\x00-\x7f]';
	my $byte2 = '[\xC0-\xDF][\x80-\xBF]';
	my $byte3 = '[\xE0-\xEF][\x80-\xBF]{2}';
	my $byte4 = '[\xF0-\xF7][\x80-\xBF]{3}';
	
	# 検索処理
	my @log;
	open(IN,"$cf{logfile}") or error("open err: $cf{logfile}");
	while (<IN>) {
		my ($no,$date,$nam,$eml,$sub,$com,$url,$hos,$pw,$tim,$ex,$w,$h,$chk) = split(/<>/);
		
		my $flg;
		foreach my $wd (@wd) {
			if ("$nam $eml $sub $com $url" =~ /^(?:$byte1|$byte2|$byte3|$byte4)*?\Q$wd\E/i) {
				$flg++;
				if ($cond == 0) { last; }
			} else {
				if ($cond == 1) { $flg = 0; last; }
			}
		}
		next if (!$flg);
		
		push(@log,$_);
	}
	close(IN);
	
	# 検索結果
	return @log;
}

#-----------------------------------------------------------
#  検索条件プルダウン
#-----------------------------------------------------------
sub op_cond {
	# 検索条件プルダウン
	my %op = (1 => 'AND', 0 => 'OR');
	my $ret;
	for (1,0) {
		if ($in{cond} eq $_) {
			$ret .= qq|<option value="$_" selected>$op{$_}</option>\n|;
		} else {
			$ret .= qq|<option value="$_">$op{$_}</option>\n|;
		}
	}
	return $ret;
}

#-----------------------------------------------------------
#  添付アップロード
#-----------------------------------------------------------
sub upload {
	my $no = shift;

	# サムネイル機能
	require './lib/thumb.pl' if ($cf{thumbnail});

	# 拡張子取得
	my $ext = $cgi->param_filename('upfile') =~ /(\.[^\\\/:\.]+)$/ ? $1 : error("拡張子不明");
	$ext =~ tr/A-Z/a-z/;
	if ($ext eq '.jpeg') { $ext = '.jpg'; }

	# ファイルチェック
	check_upl($ext);

	# 添付ファイル定義
	my $upfile = "$cf{imgdir}/$no$ext";

	# アップロード書き込み
	open(UP,"+> $upfile") or error("up err: $upfile");
	binmode(UP);
	print UP $in{upfile};
	close(UP);

	chmod(0666, $upfile);

	# 画像サイズ取得
	my ($flg,$w,$h);
	if ($ext eq ".jpg") { ($w,$h) = &j_size($upfile); $flg++; }
	elsif ($ext eq ".gif") { ($w,$h) = &g_size($upfile); $flg++; }
	elsif ($ext eq ".png") { ($w,$h) = &p_size($upfile); $flg++; }

	# サムネイル化
	if ($flg && $cf{thumbnail}) {
		($w,$h) = resize($w,$h);
		my $thumb = "$cf{imgdir}/$no-s$ext";
		make_thumb($upfile,$thumb,$w,$h);
	}

	# 結果を返す
	return ($ext,$w,$h);
}

#-----------------------------------------------------------
#  JPEGサイズ認識
#-----------------------------------------------------------
sub j_size {
	my $jpg = shift;

	my ($h, $w, $t);
	open(IMG,"$jpg");
	binmode(IMG);
	read(IMG, $t, 2);
	while (1) {
		read(IMG, $t, 4);
		my ($m, $c, $l) = unpack("a a n", $t);

		if ($m ne "\xFF") {
			$w = $h = 0;
			last;
		} elsif ((ord($c) >= 0xC0) && (ord($c) <= 0xC3)) {
			read(IMG, $t, 5);
			($h, $w) = unpack("xnn", $t);
			last;
		} else {
			read(IMG, $t, ($l - 2));
		}
	}
	close(IMG);

	return ($w, $h);
}

#-----------------------------------------------------------
#  GIFサイズ認識
#-----------------------------------------------------------
sub g_size {
	my $gif = shift;

	my $data;
	open(IMG,"$gif");
	binmode(IMG);
	sysread(IMG, $data, 10);
	close(IMG);

	if ($data =~ /^GIF/) { $data = substr($data, -4); }
	my $w = unpack("v", substr($data, 0, 2));
	my $h = unpack("v", substr($data, 2, 2));

	return ($w, $h);
}

#-----------------------------------------------------------
#  PNGサイズ認識
#-----------------------------------------------------------
sub p_size {
	my $png = shift;

	my $data;
	open(IMG,"$png");
	binmode(IMG);
	read(IMG, $data, 24);
	close(IMG);

	my $w = unpack("N", substr($data, 16, 20));
	my $h = unpack("N", substr($data, 20, 24));

	return ($w, $h);
}

#-----------------------------------------------------------
#  過去ログページ
#-----------------------------------------------------------
sub past_data {
	return if ($cf{pastkey} == 0);

	# 過去ログ番号
	open(IN,"$cf{nofile}") or error("open err: $cf{nofile}");
	my $pastnum = <IN>;
	close(IN);

	my $pastnum = sprintf("%04d", $pastnum);
	$in{pno} =~ s/\D//g;
	$in{pno} ||= $pastnum;

	# プルダウンタグ作成
	my $op_pno;
	for ( my $i = $pastnum; $i > 0; $i-- ) {
		$i = sprintf("%04d", $i);

		if ($in{pno} == $i) {
			$op_pno .= qq|<option value="$i" selected>$i\n|;
		} else {
			$op_pno .= qq|<option value="$i">$i\n|;
		}
	}

	# ページ数定義
	my $pg = $in{pg} || 0;

	# 初期化
	my ($hit,$page_btn,@log);

	# 対象ログ定義
	my $file = "$cf{pastdir}/" . sprintf("%04d", $in{pno}) . ".cgi";

	# ワード検索
	if ($in{find} && $in{word} ne '') {

		# 検索
		($hit,@log) = search($file,$in{list},'past');

		# 結果
		$page_btn = "検索結果：<b>$hit</b>件 &nbsp;&nbsp;" . pgbtn_old($hit,$in{pno},$pg,$in{list},'find');

	# ログ一覧
	} else {

		# 過去ログオープン
		my $i = 0;
		open(IN,"$file") or error("open err: $file");
		while(<IN>) {
			$i++;
			next if ($i < $pg + 1);
			next if ($i > $pg + $cf{pg_max});

			push(@log,$_);
		}
		close(IN);

		# 繰越ボタン作成
		$page_btn = pgbtn_old($i,$in{pno},$pg);
	}

	# プルダウン作成（検索条件）
	my %op = make_op();

	# テンプレート読み込み
	my ($flg,$loop);
	open(IN,"$cf{tmpldir}/past.html") or error("open err: past.html");
	my $tmpl = join('', <IN>);
	close(IN);

	# テンプレート分割
	my ($head,$loop,$foot) = $tmpl =~ m|(.+)<!-- loop -->(.+?)<!-- /loop -->(.+)|s
			? ($1,$2,$3) : error("テンプレート不正");

	if ($in{change}) { $in{word} = ''; }

	# 文字置換
	foreach ($head,$foot) {
		s/!past_num!/$in{pno}/g;
		s/!bbs_url!/$cf{html_url}\/index.html/g;
		s/!([a-z]+_cgi)!/$cf{$1}/g;
		s/<!-- op_pno -->/$op_pno/g;
		s/<!-- op_(\w+) -->/$op{$1}/g;
		s/!word!/$in{word}/g;
		s/!page_btn!/$page_btn/g;
		s/!bbs_title!/$cf{bbs_title}/g;
		s/!cmnurl!/$cf{cmnurl}/g;
	}

	# 画面表示
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;
	foreach (@log) {
		my ($no,$date,$name,$eml,$sub,$com,$url,$host,$pw,$tim,$ex,$w,$h,$chk) = split(/<>/);
		$name = qq|<a href="mailto:$eml">$name</a>| if ($eml);
		$com = autolink($com) if ($cf{autolink});
		$com =~ s/([>]|^)(&gt;[^<]*)/$1<span style="color:$cf{ref_col}">$2<\/span>/g if ($cf{ref_col});
		$com .= qq|<p class="url"><a href="$url" target="_blank"><img src="$cf{ico_home}" class="icon"></a></p>| if ($url);

		# 添付あり
		my $clip;
		if ($ex) {
			my $size = get_size("$cf{imgdir}/$no$ex");
			if ($cf{imgcheck} && $chk != 1) {
				$clip = qq|<div class="clip">[添付]: 認証待ち (<b style="color:#FF9966">$size</b>)</div>|;
			} else {
				$clip = qq|<div class="clip">[添付]: <a href="$cf{imgurl}/$no$ex" data-lightbox="img$no" data-title="$sub">$no$ex</a> (<b style="color:#FF9966">$size</b>)</div>|;
			}
		}

		my $tmp = $loop;
		$tmp =~ s/!num!/$no/g;
		$tmp =~ s/!sub!/$sub/g;
		$tmp =~ s/!date!/$date/g;
		$tmp =~ s/!name!/$name/g;
		$tmp =~ s/!comment!/$com/g;
		$tmp =~ s/<!-- clip -->/$clip/;
		print $tmp;
	}

	# フッタ
	print footer($foot);
	exit;
}

#-----------------------------------------------------------
#  プルダウン作成 [ 検索条件 ]
#-----------------------------------------------------------
sub make_op {
	my %op;
	my %cond = (1 => 'AND', 0 => 'OR');
	foreach (1,0) {
		if ($in{cond} eq $_) {
			$op{cond} .= qq|<option value="$_" selected>$cond{$_}\n|;
		} else {
			$op{cond} .= qq|<option value="$_">$cond{$_}\n|;
		}
	}
	for ( my $i = 10; $i <= 30; $i += 5 ) {
		if ($in{list} == $i) {
			$op{list} .= qq|<option value="$i" selected>$i件\n|;
		} else {
			$op{list} .= qq|<option value="$i">$i件\n|;
		}
	}
	return %op;
}

#-----------------------------------------------------------
#  アルバム機能
#-----------------------------------------------------------
sub album {
	# ページ数定義
	my $pg = $in{pg} || 0;

	# 画像サイズ再定義
	$cf{max_img_w} = $cf{alb_img_w};
	$cf{max_img_h} = $cf{alb_img_h};

	# テンプレート認識
	open(IN,"$cf{tmpldir}/album.html") or error("open err: album.html");
	my $tmpl = join('', <IN>);
	close(IN);

	$tmpl =~ s|!bbs_css!|$cf{cmnurl}/aska.css|g;
	$tmpl =~ s|!bbs_js!|$cf{cmnurl}/bbs.js|g;

	# テンプレート分割
	my ($head,$loop,$foot) = split(/<!-- loop -->/,$tmpl);

	# データ読み込み
	my ($i,@img);
	open(IN,"$cf{logfile}") or error("open err: $cf{logfile}");
	while (<IN>) {
		my ($no,$date,$name,$eml,$sub,$com,$url,$host,$pw,$tim,$ex,$w,$h,$chk) = split(/<>/);
		next if ($cf{imgcheck} && $chk eq '0');
		next if ($ex !~ /(jpg|png|gif)$/);

		$i++;
		next if ($i < $pg + 1);
		next if ($i > $pg + $cf{max_albums});

		# 画像データ収集
		push(@img,"$no\t$sub\t$ex\t$w\t$h");
	}
	close(IN);

	# 繰越ボタン
	my $page_btn = album_pgbtn($i,$pg,'&mode=album',$cf{max_albums});

	# 文字置換
	for ($head, $foot) {
		s/!([a-z]+_cgi)!/$cf{$1}/g;
		s/!page_btn!/$page_btn/g;
		s/!bbs_title!/$cf{bbs_title}/g;
		s/!cmnurl!/$cf{cmnurl}/g;

	# 過去ログ
		s/<!-- past -->.+<!-- past -->//s if ($cf{pastkey} == 0);
	}

	# 画面展開
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;

	foreach (@img) {
		my ($no,$sub,$ex,$w,$h) = split(/\t/);

		my $tmp = $loop;
		$tmp =~ s/!num!/$no/g;
		$tmp =~ s/!image!/image($no,$ex,$w,$h,$sub)/eg;
		$tmp =~ s/!caption!/$sub/g;
		print $tmp;
	}

	# フッタ
	footer($foot);
}

#-----------------------------------------------------------
#  アルバム機能 (過去ログ)
#-----------------------------------------------------------
sub album_past {
	# ページ数定義
	my $pg = $in{pg} || 0;

	# 画像サイズ再定義
	$cf{max_img_w} = $cf{alb_img_w};
	$cf{max_img_h} = $cf{alb_img_h};

	# テンプレート認識
	open(IN,"$cf{tmpldir}/album_past.html") or error("open err: album_past.html");
	my $tmpl = join('', <IN>);
	close(IN);

	$tmpl =~ s|!bbs_css!|$cf{cmnurl}/aska.css|g;
	$tmpl =~ s|!bbs_js!|$cf{cmnurl}/bbs.js|g;

	# テンプレート分割
	my ($head,$loop,$foot) = split(/<!-- loop -->/,$tmpl);

	# 過去ログ番号
	open(IN,"$cf{nofile}") or error("open err: $cf{nofile}");
	my $pastnum = <IN>;
	close(IN);

	my $pastnum = sprintf("%04d", $pastnum);
	$in{pno} =~ s/\D//g;
	$in{pno} ||= $pastnum;

	# 対象ログ定義
	my $file = "$cf{pastdir}/" . sprintf("%04d", $in{pno}) . ".cgi";

	# データ読み込み
	my ($i,@img);
	open(IN,"$file") or error("open err: $file");
	while (<IN>) {
		my ($no,$date,$name,$eml,$sub,$com,$url,$host,$pw,$tim,$ex,$w,$h,$chk) = split(/<>/);
		next if ($cf{imgcheck} && $chk eq '0');
		next if ($ex !~ /(jpg|png|gif)$/);

		$i++;
		next if ($i < $pg + 1);
		next if ($i > $pg + $cf{max_albums});

		# 画像データ収集
		push(@img,"$no\t$sub\t$ex\t$w\t$h");
	}
	close(IN);

	# 繰越ボタン
	my $page_btn = album_pgbtn($i,$pg,'&mode=album_past',$cf{max_albums});

	# 文字置換
	for ($head, $foot) {
		s/!([a-z]+_cgi)!/$cf{$1}/g;
		s/!page_btn!/$page_btn/g;
		s/!bbs_title!/$cf{bbs_title}/g;
		s/!cmnurl!/$cf{cmnurl}/g;

	}

	# 画面展開
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;

	foreach (@img) {
		my ($no,$sub,$ex,$w,$h) = split(/\t/);

		my $tmp = $loop;
		$tmp =~ s/!num!/$no/g;
		$tmp =~ s/!image!/image($no,$ex,$w,$h,$sub)/eg;
		$tmp =~ s/!caption!/$sub/g;
		print $tmp;
	}

	# フッタ
	footer($foot);
}

#-----------------------------------------------------------
#  繰越ボタン作成 (アルバム)
#-----------------------------------------------------------
sub album_pgbtn {
	my ($i,$pg,$stat,$max) = @_;
	$max ||= 5;

	# ページ繰越定義
	my $next = $pg + $max;
	my $back = $pg - $max;

	# ページ繰越ボタン作成
	my @pg;
	if ($back >= 0 || $next < $i) {
		my $flg;
		my ($w,$x,$y,$z) = (0,1,0,$i);
		while ($z > 0) {
			if ($pg == $y) {
				$flg++;
				push(@pg,qq!<li><span>$x</span></li>\n!);
			} else {
				push(@pg,qq!<li><a href="$cf{bbs_cgi}?mode=album&pg=$y">$x</a></li>\n!);
			}
			$x++;
			$y += $max;
			$z -= $max;
			
			if ($flg) { $w++; }
			last if ($w >= 5 && @pg >= 10);
		}
	}
	while( @pg >= 11 ) { shift(@pg); }
	my $ret = join('', @pg);
	if ($back >= 0) {
		$ret = qq!<li><a href="$cf{bbs_cgi}?mode=album&pg=$back">&laquo;</a></li>\n! . $ret;
	}
	if ($next < $i) {
		$ret .= qq!<li><a href="$cf{bbs_cgi}?mode=album&pg=$next">&raquo;</a></li>\n!;
	}
	
	# 結果を返す
	return $ret ? qq|<ul class="pager">\n$ret</ul>| : '';
}

#-----------------------------------------------------------
#  画像表示
#-----------------------------------------------------------
sub image {
	my ($no,$ex,$w,$h,$sub) = @_;

	my $image;
	if (-f "$cf{imgdir}/$no-s$ex") {
		$image = qq|<img src="$cf{imgdir}/$no-s$ex" border="0">|;

	} else {
		($w,$h) = resize($w,$h);
		$image = qq|<img src="$cf{imgdir}/$no$ex" width="$w" height="$h" border="0">|;
	}
	return qq|<a href="$cf{imgdir}/$no$ex" data-lightbox="aska-set" data-title="$sub">$image</a>\n|;
}

#-----------------------------------------------------------
#  留意事項
#-----------------------------------------------------------
sub note_page {
	open(IN,"$cf{tmpldir}/note.html") or error("open err: note.html");
	my $tmpl = join('',<IN>);
	close(IN);
	
	$tmpl =~ s/!cmnurl!/$cf{cmnurl}/g;
	$tmpl =~ s/!bbs_title!/$cf{bbs_title}/g;
	$tmpl =~ s|!icon:(\w+\.\w+)!|<img src="$cf{cmnurl}/$1" class="icon" alt="">|g;
	
	print "Content-type: text/html; charset=utf-8\n\n";
	print $tmpl;
	exit;
}

#-----------------------------------------------------------
#  ユーザ記事削除
#-----------------------------------------------------------
sub dele_data {
	$in{del} =~ s/\D//g;
	
	my ($flg,$crypt,@log);
	open(DAT,"+< $cf{logfile}");
	eval "flock(DAT,2);";
	while (<DAT>) {
		my ($no,$dat,$nam,$eml,$sub,$com,$url,$hos,$pw,$tim) = split(/<>/);
		
		if ($in{del} == $no) {
			$flg++;
			$crypt = $pw;
			next;
		}
		push(@log,$_);
	}
	
	if (!$flg or !$crypt) {
		close(DAT);
		error("不正な要求です");
	}
	
	# 削除キーを照合
	if (decrypt($in{pwd},$crypt) != 1) {
		close(DAT);
		error("認証できません");
	}
	
	# ログ更新
	seek(DAT,0,0);
	print DAT @log;
	truncate(DAT,tell(DAT));
	close(DAT);
	
	# 完了
	message("記事を削除しました");
}

#-----------------------------------------------------------
#  削除フォーム
#-----------------------------------------------------------
sub del_form {
	$in{del} =~ s/\D//g;
	my ($flg,$sub);
	open(DAT,"$cf{logfile}");
	while (<DAT>) {
		my ($no,$dat,$nam,$eml,$sb,$com,$url,$hos,$pw,$tim) = split(/<>/);
		
		if ($in{del} == $no) {
			$flg = $pw eq '' ? 1 : 2;
			$sub = $sb;
			last;
		}
	}
	close(IN);
	
	if (!$flg) { error('不正な要求です'); }
	elsif ($flg == 1) { error('この記事は削除できません'); }
	
	open(IN,"$cf{tmpldir}/del.html") or error('open err: del.html');
	my $tmpl = join('',<IN>);
	close(IN);
	
	$tmpl =~ s/!del!/$in{del}/g;
	$tmpl =~ s/!sub!/$sub/;
	$tmpl =~ s/!(bbs_cgi|cmnurl|bbs_title)!/$cf{$1}/g;
	$tmpl =~ s|!icon:(\w+\.\w+)!|<img src="$cf{cmnurl}/$1" class="icon" alt="$1">|g;
	$tmpl =~ s/!pg!/$in{pg} eq '' ? '0' : $in{pg}/e;
	
	print "Content-type: text/html; charset=utf-8\n\n";
	print $tmpl;
	exit;
}

#-----------------------------------------------------------
#  ユーザ記事修正
#-----------------------------------------------------------
sub edit_data {
	# チェック
	if ($in{edit} =~ /\D/) { error('不正な要求です'); }

	# 修正フォーム
	if (!$in{pwd}) { gate_form(); }

	# 実行
	if ($in{job} eq "edit") {

	# 投稿チェック
	check_post();

	# 差し替え
	my ($flg, $ext2,$w2,$h2, @data);
	open(DAT,"+< $cf{logfile}") || &error("Open Error: $cf{logfile}");
	eval "flock(DAT, 2);";
	while (<DAT>) {
		my ($no,$date,$name,$eml,$sub,$com,$url,$host,$pw,$tim,$ext,$w,$h) = split(/<>/);

		if ($in{edit} == $no) {
		         $flg = 1;

			# アップロード
			if ($in{upfile}) {
				($ext2,$w2,$h2) = upload($in{edit});
			}

			# 画像削除
			if ($in{imgdel}) {
				unlink("$cf{imgdir}/$in{edit}$ext");
				$ext = $w = $h = '';
			}

			# 添付アップロードの場合
			if ($ext2) {
				# 拡張子変更の場合旧ファイル削除
				if ($ext && $ext2 ne $ext) {
					unlink("$cf{imgdir}/$in{edit}$ext");
				}
				# 新添付ファイル情報
				($ext,$w,$h) = ($ext2,$w2,$h2);
			}

		         $_ = "$no<>$date<>$in{name}<>$in{email}<>$in{sub}<>$in{comment}<>$in{url}<>$host<>$pw<>$tim<>$ext<>$w<>$h<>0<>\n";
	}
	push(@data,$_);
	}

	if ($flg != 1) {
		close(DAT);
		error("不正な処理です");
	}

	# 更新
	seek(DAT, 0, 0);
	print DAT @data;
	truncate(DAT, tell(DAT));
	close(DAT);

	# 完了メッセージ
	message("記事の修正を完了しました");
}

	# 記事抽出
	my ($flg, @edit);
	open(IN,"$cf{logfile}") || &error("Open Error: $cf{logfile}");
	while (<IN>) {
		my ($no,$date,$name,$eml,$sub,$com,$url,$host,$pw,$tim,$ext,$w,$h) = split(/<>/);

		if ($in{edit} == $no) {

			# パスワードなし
			if ($pw eq '') {
				$flg = 2;
			} elsif

			# パスワード一致
			(&decrypt($in{pwd}, $pw) == 1) {
				$flg = 1;
				@edit = ($no,$date,$name,$eml,$sub,$com,$url,$host,$pw,$tim,$ext,$w,$h);

			# パスワード不一致
			} else {
				$flg = 3;
			}
			last;
		}
	}
	close(IN);

	if (!$flg) {
		close(DAT);
		error("該当記事が見当たりません");
	} elsif ($flg == 2) {
		close(DAT);
		error("この記事にはパスワードが設定されていません");
	} elsif ($flg == 3) {
		close(DAT);
		error("パスワードが違います");
	}

	# 改行復元
	my ($no,$date,$name,$eml,$sub,$com,$url,$host,$pw,$tim,$ext) = split(/<>/);
	$com =~ s/<br>/\n/g;
	$url ||= 'http://';

	# 添付ファイル
	my $attach;
	if ($ext) {
		$attach .= qq|&nbsp;[ <a href="$cf{imgdir}/$no$ext" target="_blank">Attached file</a> ]\n|;
		$attach .= qq|<input type="checkbox" name="imgdel" id="checkbox" value="1"><label for="checkbox">削除</label>\n|;
	}

	# 修正フォーム
	open(IN,"$cf{tmpldir}/edit.html") or &error("open err: edit.html");
	my $tmpl = join('', <IN>);
	close(IN);

	$tmpl =~ s/!([a-z]+_cgi)!/$cf{$1}/g;
	$tmpl =~ s/!bbs_title!/$cf{bbs_title}/g;
	$tmpl =~ s/!cmnurl!/$cf{cmnurl}/g;
	$tmpl =~ s/!edit!/$in{edit}/g;
	$tmpl =~ s/!num!/$no/g;
	$tmpl =~ s/!pwd!/$pw/g;
	$tmpl =~ s/!name!/$name/;
	$tmpl =~ s/!email!/$eml/;
	$tmpl =~ s/!url!/$url/;
	$tmpl =~ s/!sub!/$sub/;
	$tmpl =~ s/!comment!/$com/;
	$tmpl =~ s/<!-- attach -->/$attach/g;

	print "Content-type: text/html; charset=utf-8\n\n";
	print $tmpl;

	# フッタ
	footer();

	exit;
}

#-----------------------------------------------------------
#  暗証キー入力
#-----------------------------------------------------------
sub gate_form {

	# 記事
	my ($name,$pwd);
	open(DAT,"+< $cf{logfile}") || &error("Open Error: $cf{logfile}");
	eval "flock(DAT, 2);";
	while (<DAT>) {
		my ($no,$date,$name,$eml,$sub,$com,$url,$host,$pw,$tim,$ex,$w,$h,$chk) = split(/<>/);

	if ($in{edit} == $no) {
		$name = $name;
		$pwd = $pw;
		last;
		}
	}
	close(DAT);

	# パスワード未設定
	error("この記事はパスワード未設定のため編集できません") if ($pwd eq '');

	my ($no,$date,$name,$eml,$sub,$com,$url,$host,$pw,$tim,$ex,$w,$h,$chk) = split(/<>/);
		$name = qq|<a href="mailto:$eml">$name</a>| if ($eml);
		$com = autolink($com) if ($cf{autolink});
		$com =~ s/([>]|^)(&gt;[^<]*)/$1<span style="color:$cf{ref_col}">$2<\/span>/g if ($cf{ref_col});

		if (length($com) > 135) { $com = substr($com,0,145); $com .= '.....(略)'; }

		# 添付あり
		my $clip;
		if ($ex) {
			my $size = get_size("$cf{imgdir}/$no$ex");
			if ($cf{imgcheck} && $chk != 1) {
				$clip = qq|<div class="clip">[添付]: 認証待ち (<b style="color:#FF9966">$size</b>)</div>|;
			} else {
				$clip = qq|<div class="clip">[添付]: <a href="$cf{imgurl}/$no$ex" target="_blank">$no$ex</a> (<b style="color:#FF9966">$size</b>)</div>|;
			}
		}

	open(IN,"$cf{tmpldir}/gate.html") or &error("open err: gate.html");
	my $tmpl = join('', <IN>);
	close(IN);

	# 文字置換
	$tmpl =~ s/!edit!/$in{edit}/g;
	$tmpl =~ s/!bbs_title!/$cf{bbs_title}/g;
	$tmpl =~ s/!num!/$no/g;
	$tmpl =~ s/!sub!/$sub/g;
	$tmpl =~ s/!name!/$name/g;
	$tmpl =~ s/!date!/$date/g;
	$tmpl =~ s/!comment!/$com/g;
	$tmpl =~ s/!home!/home_link($url)/e;
	$tmpl =~ s/!bbs_cgi!/$cf{bbs_cgi}/g;
	$tmpl =~ s/<!-- clip -->/$clip/;
	$tmpl =~ s/!cmnurl!/$cf{cmnurl}/g;
	$tmpl =~ s|!icon:(\w+\.\w+)!|<img src="$cf{cmnurl}/$1" alt="" class="icon" />|g;

	print "Content-type: text/html; charset=utf-8\n\n";
	print $tmpl;

	# フッタ
	footer();

	exit;
}

#-----------------------------------------------------------
#  エラー画面
#-----------------------------------------------------------
sub error {
	my ($err,$btn) = @_;
	
	open(IN,"$cf{tmpldir}/error.html") or die;
	my $tmpl = join('',<IN>);
	close(IN);
	
	$tmpl =~ s/!error!/$err/g;
	$tmpl =~ s/!(bbs_cgi|cmnurl|bbs_title)!/$cf{$1}/g;
	$tmpl =~ s|!icon:(\w+\.\w+)!|<img src="$cf{cmnurl}/$1" class="icon" alt="$1">|g;
	
	if ($btn) {
		$tmpl =~ s|<!-- button -->.+?<!-- /button -->|$btn|s;
	}
	
	print "Content-type: text/html; charset=utf-8\n\n";
	print $tmpl;
	exit;
}

#-----------------------------------------------------------
#  メール送信
#-----------------------------------------------------------
sub mail_to {
	my ($date,$host,$clip) = @_;
	
	# 件名をMIMEエンコード
	require "lib/jacode.pl";
	my $msub = mime_unstructured_header("BBS : $in{sub}");
	
	# コメント内の改行復元
	my $com = $in{comment};
	$com =~ s|<br>|\n|g;
	$com =~ s/&lt;/>/g;
	$com =~ s/&gt;/</g;
	$com =~ s/&quot;/"/g;
	$com =~ s/&amp;/&/g;
	$com =~ s/&#39;/'/g;
	
	# メール本文を定義
	my $body = <<EOM;
$cf{bbs_title}に投稿がありました。

投稿日：$date
ホスト：$host

件名  ：$in{sub}
お名前：$in{name}
E-mail：$in{email}
URL   ：$in{url}
添付：$clip

$com
EOM

	# JISコード変換
	my $mbody;
	for my $tmp ( split(/\n/,$body) ) {
		jcode::convert(\$tmp,'jis','utf8');
		$mbody .= "$tmp\n";
	}
	
	# FROM行の定義
	my $from = $in{email} || $cf{mailto};
	
	# sendmailコマンド
	my $scmd = "$cf{sendmail} -t -i";
	if ($cf{sendm_f} == 1) { $scmd .= " -f $from"; }
	
	# 送信
	open(MAIL,"| $scmd") or error("送信失敗");
	print MAIL "To: $cf{mailto}\n";
	print MAIL "From: $from\n";
	print MAIL "Subject: $msub\n";
	print MAIL "MIME-Version: 1.0\n";
	print MAIL "Content-type: text/plain; charset=ISO-2022-JP\n";
	print MAIL "Content-Transfer-Encoding: 7bit\n";
	print MAIL "X-Mailer: $cf{version}\n\n";
	print MAIL "$mbody\n";
	close(MAIL);
}

#-----------------------------------------------------------
#  フッター
#-----------------------------------------------------------
sub footer {
	my $foot = shift;
	
	# 著作権表記（削除・改変禁止）
	my $copy = <<EOM;
<p style="margin-top:2em;text-align:center;font-family:Verdana,Helvetica,Arial;font-size:11pt;">
	- <a href="https://www.kent-web.com/" target="_top">$cf{version}</a> -<br />
	++ <a href="https://www.transformsite.com/" target="_top">Edited by Hamel</a> ++
</p>
EOM

	if ($foot =~ /(.+)(<\/body[^>]*>.*)/si) {
		print "$1$copy$2\n";
	} else {
		print "$foot$copy\n";
		print "</body></html>\n";
	}
	exit;
}

#-----------------------------------------------------------
#  自動リンク
#-----------------------------------------------------------
sub autolink {
	my $text = shift;
	
	$text =~ s/(s?https?:\/\/([\w-.!~*'();\/?:\@=+\$,%#]|&amp;)+)/<a href="$1" target="_blank">$1<\/a>/g;
	return $text;
}

#-----------------------------------------------------------
#  禁止ワードチェック
#-----------------------------------------------------------
sub no_wd {
	my $flg;
	foreach ( split(/,/,$cf{no_wd}) ) {
		if (index("$in{name} $in{sub} $in{comment}", $_) >= 0) {
			$flg = 1;
			last;
		}
	}
	if ($flg) { error("禁止ワードが含まれています"); }
}

#-----------------------------------------------------------
#  URL個数チェック
#-----------------------------------------------------------
sub urlnum {
	my $com = $in{comment};
	my ($num) = ($com =~ s|(https?://)|$1|ig);
	if ($num > $cf{urlnum}) {
		error("コメント中のURLアドレスは最大$cf{urlnum}個までです");
	}
}

#-----------------------------------------------------------
#  アクセス制限
#-----------------------------------------------------------
sub get_host {
	# IP&ホスト取得
	my $host = $ENV{REMOTE_HOST};
	my $addr = $ENV{REMOTE_ADDR};
	if ($cf{gethostbyaddr} && ($host eq "" || $host eq $addr)) {
		$host = gethostbyaddr(pack("C4", split(/\./, $addr)), 2);
	}
	
	# IPチェック
	my $flg;
	foreach ( split(/\s+/,$cf{deny_addr}) ) {
		s/\./\\\./g;
		s/\*/\.\*/g;
		
		if ($addr =~ /^$_/i) { $flg++; last; }
	}
	if ($flg) {
		error("アクセスを許可されていません");
	
	# ホストチェック
	} elsif ($host) {
		
		for ( split(/\s+/,$cf{deny_host}) ) {
			s/\./\\\./g;
			s/\*/\.\*/g;
			
			if ($host =~ /$_$/i) { $flg++; last; }
		}
		if ($flg) {
			error("アクセスを許可されていません");
		}
	}
	
	if ($host eq "") { $host = $addr; }
	return ($host,$addr);
}

#-----------------------------------------------------------
#  crypt暗号
#-----------------------------------------------------------
sub encrypt {
	my $in = shift;
	
	my @wd = (0 .. 9, 'a'..'z', 'A'..'Z', '.', '/');
	my $salt = $wd[int(rand(@wd))] . $wd[int(rand(@wd))];
	crypt($in,$salt) || crypt ($in,'$1$'.$salt);
}

#-----------------------------------------------------------
#  crypt照合
#-----------------------------------------------------------
sub decrypt {
	my ($in,$dec) = @_;
	
	my $salt = $dec =~ /^\$1\$(.*)\$/ ? $1 : substr($dec,0,2);
	if (crypt($in,$salt) eq $dec || crypt($in,'$1$'.$salt) eq $dec) {
		return 1;
	} else {
		return 0;
	}
}

#-----------------------------------------------------------
#  完了メッセージ
#-----------------------------------------------------------
sub message {
	my $msg = shift;
	
	open(IN,"$cf{tmpldir}/mesg.html") or error("open err: mesg.html");
	my $tmpl = join('',<IN>);
	close(IN);
	
	$tmpl =~ s/!(bbs_cgi|cmnurl|bbs_title)!/$cf{$1}/g;
	$tmpl =~ s/!message!/$msg/g;
	$tmpl =~ s|!icon:(\w+\.\w+)!|<img src="$cf{cmnurl}/$1" class="icon" alt="">|g;
	
	print "Content-type: text/html; charset=utf-8\n\n";
	print $tmpl;
	exit;
}

#-----------------------------------------------------------
#  繰越ボタン作成 [ 過去ログ ]
#-----------------------------------------------------------
sub pgbtn_old {
	my ($i,$pno,$pg,$stat) = @_;

	# ページ繰越定義
	$cf{pg_max} ||= 10;
	my $next = $pg + $cf{pg_max};
	my $back = $pg - $cf{pg_max};

	my $link;
	if ($stat eq 'past') {
		my $wd = url_enc($in{word});
		$link = "$cf{bbs_cgi}?mode=$in{mode}&amp;pno=$pno&amp;find=1&amp;word=$wd";
	} else {
		$link = "$cf{bbs_cgi}?mode=$in{mode}&amp;pno=$pno";
	}

	# ページ繰越ボタン作成
	my @pg;
	if ($back >= 0 || $next < $i) {
		my $flg;
		my ($w,$x,$y,$z) = (0,1,0,$i);
		while ($z > 0) {
			if ($pg == $y) {
				$flg++;
				push(@pg,qq!<li><span>$x</span></li>\n!);
			} else {
				push(@pg,qq!<li><a href="$link&amp;pg=$y">$x</a></li>\n!);
			}
			$x++;
			$y += $cf{pg_max};
			$z -= $cf{pg_max};
			
			if ($flg) { $w++; }
			last if ($w >= 5 && @pg >= 10);
		}
	}
	while( @pg >= 11 ) { shift(@pg); }
	my $ret = join('', @pg);
	if ($back >= 0) {
		$ret = qq!<li><a href="$link&amp;pg=$back">&laquo;</a></li>\n! . $ret;
	}
	if ($next < $i) {
		$ret .= qq!<li><a href="$link&amp;pg=$next">&raquo;</a></li>\n!;
	}
	
	# 結果を返す
	return $ret ? qq|<ul class="pager">\n$ret</ul>| : '';
}

#-----------------------------------------------------------
#  ページ送り作成
#-----------------------------------------------------------
sub make_pager {
	my ($i,$pg) = @_;

	# ページ繰越数定義
	$cf{pg_max} ||= 10;
	my $next = $pg + $cf{pg_max};
	my $back = $pg - $cf{pg_max};

	# ページ繰越ボタン作成
	my @pg;
	if ($back >= 0 || $next < $i) {
		my $flg;
		my ($w,$x,$y,$z) = (0,1,0,$i);
		while ($z > 0) {
			if ($pg == $y) {
				$flg++;
				push(@pg,qq!<li><span>$x</span></li>\n!);
			} else {
				push(@pg,qq!<li><a href="$cf{bbs_cgi}?pg=$y">$x</a></li>\n!);
			}
			$x++;
			$y += $cf{pg_max};
			$z -= $cf{pg_max};
			
			if ($flg) { $w++; }
			last if ($w >= 5 && @pg >= 10);
		}
	}
	while( @pg >= 11 ) { shift(@pg); }
	my $ret = join('', @pg);
	if ($back >= 0) {
		$ret = qq!<li><a href="$cf{bbs_cgi}?pg=$back">&laquo;</a></li>\n! . $ret;
	}
	if ($next < $i) {
		$ret .= qq!<li><a href="$cf{bbs_cgi}?pg=$next">&raquo;</a></li>\n!;
	}
	
	# 結果を返す
	return $ret ? qq|<ul class="pager">\n$ret</ul>| : '';
}

#-----------------------------------------------------------
#  カウンタ処理
#-----------------------------------------------------------
sub bbs_count {
	# IP取得
	my $addr = $ENV{REMOTE_ADDR};

	# 閲覧時のみカウントアップ
	my $cntup = $in{mode} eq '' ? 1 : 0;

	# カウントファイルを読みこみ
	open(LOG,"+< $cf{cntfile}") or error("open err: $cf{cntfile}");
	eval "flock(LOG, 2);";
	my $count = <LOG>;

	# IPチェックとログ破損チェック
	my ($cnt,$ip) = split(/:/,$count);
	if ($addr eq $ip or $cnt eq "") { $cntup = 0; }

	# カウントアップ
	if ($cntup) {
		$cnt++;
		seek(LOG, 0, 0);
		print LOG "$cnt:$addr";
		truncate(LOG, tell(LOG));
	}
	close(LOG);

	# 桁数調整
	while(length($cnt) < $cf{mini_fig}) { $cnt = '0' . $cnt; }
	my @cnts = split(//,$cnt);

	# GIFカウンタ表示
	my $counter;
	if ($cf{counter} == 2) {
		foreach (0 .. $#cnts) {
			$counter .= qq|<img src="$cf{gif_path}/$cnts[$_].gif\" alt="$cnts[$_]" />|;
		}

	# テキストカウンタ表示
	} else {
		$counter = qq|<span style="color:$cf{cntcol}">$cnt</span>\n|;
	}
	return $counter;
}

#-----------------------------------------------------------
#  クッキー発行
#-----------------------------------------------------------
sub set_cookie {
	my @data = @_;
	
	# 60日間有効
	my ($sec,$min,$hour,$mday,$mon,$year,$wday,undef,undef) = gmtime(time + 60*24*60*60);
	my @mon  = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);
	my @week = qw(Sun Mon Tue Wed Thu Fri Sat);
	
	# 時刻フォーマット
	my $gmt = sprintf("%s, %02d-%s-%04d %02d:%02d:%02d GMT",
				$week[$wday],$mday,$mon[$mon],$year+1900,$hour,$min,$sec);
	
	# URLエンコード
	my $cook;
	foreach (@data) {
		s/(\W)/sprintf("%%%02X", unpack("C", $1))/eg;
		$cook .= "$_<>";
	}
	
	print "Set-Cookie: $cf{cookie_id}=$cook; expires=$gmt\n";
}

#-----------------------------------------------------------
#  クッキー取得
#-----------------------------------------------------------
sub get_cookie {
	# クッキー取得
	my $cook = $ENV{HTTP_COOKIE};
	
	# 該当IDを取り出す
	my %cook;
	foreach ( split(/;/,$cook) ) {
		my ($key,$val) = split(/=/);
		$key =~ s/\s//g;
		$cook{$key} = $val;
	}
	
	# URLデコード
	my @cook;
	foreach ( split(/<>/,$cook{$cf{cookie_id}}) ) {
		s/%([0-9A-Fa-f][0-9A-Fa-f])/pack("H2", $1)/eg;
		s/[&"'<>]//g;
		
		push(@cook,$_);
	}
	return @cook;
}

#-----------------------------------------------------------
#  ファイル容量
#-----------------------------------------------------------
sub get_size {
	my $file = shift;
	
	my $size = -s $file || 0;
	return $size > 1024000 ? comma(int($size/1024000)) . 'MB' : $size > 1024 ? comma(int($size/1024)) . 'KB' : $size . 'Bytes';
}

#-----------------------------------------------------------
#  桁区切り
#-----------------------------------------------------------
sub comma {
	local($_) = @_;

	1 while s/(.*\d)(\d\d\d)/$1,$2/;
	$_;
}

#-----------------------------------------------------------
#  ファイルチェック
#-----------------------------------------------------------
sub check_upl {
	my ($ext) = @_;

	# mimeタイプ
	my $mime = $cgi->param_mime('upfile');

	my $flg;
	if ($cf{ok_gif}) {
		if ($mime =~ /^image\/gif$/i and $ext eq '.gif') { $flg++; }
	}
	if (!$flg and $cf{ok_jpeg}) {
		if ($mime =~ /^image\/p?jpe?g$/i and $ext eq '.jpg') { $flg++; }
	}
	if (!$flg and $cf{ok_png}) {
		if ($mime =~ /^image\/(x-)?png$/i and $ext eq '.png') { $flg++; }
	}
	if (!$flg and $cf{ok_pdf}) {
		if ($mime =~ /^application\/pdf$/i and $ext eq '.pdf') { $flg++; }
	}
	if (!$flg and $cf{ok_zip}) {
		if ($mime =~ /^application\/(x-)?zip(-compressed)?$/i and $ext eq '.zip') { $flg++; }
	}
	if (!$flg and $cf{ok_text}) {
		if ($mime =~ /^text\/plain$/i and $ext =~ /^\.te?xt$/) { $flg++; }
	}
	if (!$flg and $cf{ok_word}) {
		if ($mime =~ /^application\/(vnd\.)?ms-?word$/i and $ext eq '.doc') { $flg++; }
		elsif ($mime =~ /^application\/vnd\.openxmlformats-officedocument\.wordprocessingml\.document$/i and $ext eq '.docx') { $flg++; }
	}
	if (!$flg and $cf{ok_excel}) {
		if ($mime =~ /^application\/(vnd\.)?ms-?excel$/i and $ext eq '.xls') { $flg++; }
		elsif ($mime =~ /^application\/vnd\.openxmlformats-officedocument\.spreadsheetml\.sheet$/i and $ext eq '.xlsx') { $flg++; }
	}
	if (!$flg and $cf{ok_ppt}) {
		if ($mime =~ /^application\/(vnd\.)?ms-?powerpoint$/i and $ext eq '.ppt') { $flg++; }
		elsif ($mime =~ /^application\/vnd\.openxmlformats-officedocument\.presentationml\.presentation$/i and $ext eq '.pptx') { $flg++; }
	}

	if (!$flg) { error('このファイルは取り扱いできません'); }
}

#-----------------------------------------------------------
#  mimeエンコード
#  [quote] http://www.din.or.jp/~ohzaki/perl.htm#JP_Base64
#-----------------------------------------------------------
sub mime_unstructured_header {
  my $oldheader = shift;
  jcode::convert(\$oldheader,'euc','utf8');
  my ($header,@words,@wordstmp,$i);
  my $crlf = $oldheader =~ /\n$/;
  $oldheader =~ s/\s+$//;
  @wordstmp = split /\s+/, $oldheader;
  for ($i = 0; $i < $#wordstmp; $i++) {
    if ($wordstmp[$i] !~ /^[\x21-\x7E]+$/ and
	$wordstmp[$i + 1] !~ /^[\x21-\x7E]+$/) {
      $wordstmp[$i + 1] = "$wordstmp[$i] $wordstmp[$i + 1]";
    } else {
      push(@words, $wordstmp[$i]);
    }
  }
  push(@words, $wordstmp[-1]);
  foreach my $word (@words) {
    if ($word =~ /^[\x21-\x7E]+$/) {
      $header =~ /(?:.*\n)*(.*)/;
      if (length($1) + length($word) > 76) {
	$header .= "\n $word";
      } else {
	$header .= $word;
      }
    } else {
      $header = add_encoded_word($word, $header);
    }
    $header =~ /(?:.*\n)*(.*)/;
    if (length($1) == 76) {
      $header .= "\n ";
    } else {
      $header .= ' ';
    }
  }
  $header =~ s/\n? $//mg;
  $crlf ? "$header\n" : $header;
}
sub add_encoded_word {
  my ($str, $line) = @_;
  my $result;
  my $ascii = '[\x00-\x7F]';
  my $twoBytes = '[\x8E\xA1-\xFE][\xA1-\xFE]';
  my $threeBytes = '\x8F[\xA1-\xFE][\xA1-\xFE]';
  while (length($str)) {
    my $target = $str;
    $str = '';
    if (length($line) + 22 +
	($target =~ /^(?:$twoBytes|$threeBytes)/o) * 8 > 76) {
      $line =~ s/[ \t\n\r]*$/\n/;
      $result .= $line;
      $line = ' ';
    }
    while (1) {
      my $encoded = '=?ISO-2022-JP?B?' .
      b64encode(jcode::jis($target,'euc','z')) . '?=';
      if (length($encoded) + length($line) > 76) {
	$target =~ s/($threeBytes|$twoBytes|$ascii)$//o;
	$str = $1 . $str;
      } else {
	$line .= $encoded;
	last;
      }
    }
  }
  $result . $line;
}
# [quote] http://www.tohoho-web.com/perl/encode.htm
sub b64encode {
    my $buf = shift;
    my ($mode,$tmp,$ret);
    my $b64 = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
                . "abcdefghijklmnopqrstuvwxyz"
                . "0123456789+/";
	
    $mode = length($buf) % 3;
    if ($mode == 1) { $buf .= "\0\0"; }
    if ($mode == 2) { $buf .= "\0"; }
    $buf =~ s/(...)/{
        $tmp = unpack("B*", $1);
        $tmp =~ s|(......)|substr($b64, ord(pack("B*", "00$1")), 1)|eg;
        $ret .= $tmp;
    }/eg;
    if ($mode == 1) { $ret =~ s/..$/==/; }
    if ($mode == 2) { $ret =~ s/.$/=/; }
    
    return $ret;
}

#-----------------------------------------------------------
#  Homeリンク
#-----------------------------------------------------------
sub home_link {
	my $url = shift;
	
	return $url ? qq|<a href="$url"><img src="$cf{cmnurl}/home.png" alt="home" class="icon"></a>| : '';
}

#-----------------------------------------------------------
#  入力チェック
#-----------------------------------------------------------
sub check_post {
	# 投稿チェック
	if ($cf{postonly} && $ENV{REQUEST_METHOD} ne 'POST') {
		error("不正なリクエストです");
	}
	
	# 不要改行カット
	$in{sub}  =~ s|<br>||g;
	$in{name} =~ s|<br>||g;
	$in{pwd}  =~ s|<br>||g;
	$in{comment} =~ s|(<br>)+$||g;
	
	if ($cf{no_wd}) { no_wd(); }
	if ($cf{urlnum} > 0) { urlnum(); }
	
	# 未入力の場合
	if ($in{url} eq "http://") { $in{url} = ""; }
	if ($in{sub} eq '') { $in{sub} = '無題'; }
	
	# フォーム内容をチェック
	my %err;
	if ($in{name} eq "") { $err{name} = "名前が入力されていません"; }
	if ($in{email} ne '' && $in{email} !~ /^[\w\.\-]+\@[\w\.\-]+\.[a-zA-Z]{2,6}$/) {
		$err{eml} = "e-mailの入力内容が不正です";
	}
	if ($in{comment} eq "") { $err{com} = "メッセージが入力されていません"; }
	elsif ($cf{jp_wd} && $in{comment} !~ /(?:[\xC0-\xDF][\x80-\xBF]|[\xE0-\xEF][\x80-\xBF]{2}|[\xF0-\xF7][\x80-\xBF]{3})/x) {
		$err{com} = "メッセージに日本語が含まれていません";
	}
	if ($in{url} ne '' && $in{url} !~ /^https?:\/\/[\w-.!~*'();\/?:\@&=+\$,%#]+$/i) {
		$err{url} = "参照先URLの入力内容が不正です";
	}
	if (%err > 0) { form_page(%err); }
}

#-----------------------------------------------------------
#  セッション確認
#-----------------------------------------------------------
sub check_ses {
	my $now = time;
	my ($flg,@log);
	open(DAT,"+< $cf{conffile}");
	eval "flock(DAT,2);";
	while(<DAT>) {
		chomp;
		my ($time,$rand,$fig) = split(/\t/);
		next if ($now - $time > $cf{cap_time}*60);
		
		if ($in{ses} eq $rand) {
			$flg = 1;
			if ($cf{use_captcha} > 0 && $fig ne $in{captcha}) {
				$flg = -1;
				last;
			}
			next;
		}
		push(@log,"$_\n");
	}
	if ($flg == -1) {
		close(DAT);
		error("画像認証できません");
	}
	seek(DAT,0,0);
	print DAT @log;
	truncate(DAT,tell(DAT));
	close(DAT);
	
	if (!$flg) {
		my $msg = "セッションが不正です<br>";
		$msg .= "以下をクリックして再度投稿し直してください\n";
		my $btn .= qq|<input type="button" class="color red button" onclick="window.open('$cf{bbs_cgi}','_self')" value="掲示板に戻る">|;
		error($msg,$btn);
	}
}

